using System.Diagnostics;
using System.Threading;
using MongoDB.Driver;
using Todo.Core.Interfaces.Persistence;
using Todo.Persistence.Mongo.Repositories.Dtos;

namespace Todo.Persistence.Mongo.Repositories;

public class TodoRepository : ITodoRepository
{
    private readonly IMongoCollection<MongoTodo> _todosCollection;

    public TodoRepository(MongoDbConnectionHandler mongoDbConnectionHandler)
    {
        var database = mongoDbConnectionHandler.CreateDatabaseConnection();

        _todosCollection = database.GetCollection<MongoTodo>("todos");
    }

    public async Task<Core.Entities.Todo> CreateTodoAsync(
        string title,
        string? projectName,
        string? description,
        string? priority,
        string userId,
        long created)
    {
        var todo = new MongoTodo
        {
            Title = title,
            ProjectName = projectName,
            Status = false,
            AuthorId = userId,
            Description = description,
            Created = created,
            Priority = priority
        };
        await _todosCollection.InsertOneAsync(todo);
        return todo.To();
    }

    public async Task<IEnumerable<Core.Entities.Todo>> GetTodosAsync(string userId, CancellationToken cancellationToken)
    {
        var todos = await _todosCollection.FindAsync(t => t.AuthorId == userId, cancellationToken: cancellationToken);
        return todos
            .ToEnumerable()
            .Select(t => t.To());
    }

    public async Task UpdateTodoStatus(
        string todoId,
        bool todoStatus,
        string userId)
    {
        await _todosCollection
            .UpdateOneAsync(
                t => t.Id == todoId && t.AuthorId == userId,
                Builders<MongoTodo>.Update.Set(t => t.Status, todoStatus));
    }

    public async Task<Core.Entities.Todo> GetTodoByIdAsync(string todoId)
    {
        var todoCursor = await _todosCollection.FindAsync(f => f.Id == todoId);
        var todo = await todoCursor.FirstOrDefaultAsync();

        return todo.To();
    }

    public async Task<Core.Entities.Todo> UpdateTodoAsync(
        Core.Entities.Todo todo)
    {
        var updatedTodo = await _todosCollection.FindOneAndReplaceAsync(
            f => f.Id == todo.Id,
            new MongoTodo
            {
                Id = todo.Id,
                Status = todo.Status,
                Title = todo.Title,
                ProjectName = todo.Project,
                AuthorId = todo.AuthorId,
                Description = todo.Description,
                Created = todo.Created,
                Priority = todo.Priority
            });

        return updatedTodo.To();
    }

    public async Task<IEnumerable<Core.Entities.Todo>> GetActiveTodosAsync(
        string authorId,
        CancellationToken cancellationToken)
    {
        var todosCursor = await _todosCollection.FindAsync(
            t => t.AuthorId == authorId && t.Status == false,
            cancellationToken: cancellationToken);
        var todos = todosCursor.ToEnumerable(cancellationToken);

        return todos.Select(t => t.To());
    }
}